/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package swing.addon.plaf.themelf;

import java.io.*;
import java.net.*;
import java.util.*;

public class DirectoryClassLoader extends ClassLoader {
	private static final boolean DEBUG = false;

	String dir;
	
	public DirectoryClassLoader(File directory)  {
		this(directory.toString());
	}

	public DirectoryClassLoader(String directory)  {
		dir = directory;
		
		if (DEBUG) System.out.println("DirectoryClassLoader.findClass(): directory to load from is " + dir);
	}

	protected Class findClass(String name) throws ClassNotFoundException {
		try
			{
			if (DEBUG) System.out.println("DirectoryClassLoader.findClass(): loading class " + name);
			
			File f = new File(dir, name + ".class");
			DataInputStream data = new DataInputStream(new FileInputStream(f));
		
			byte [] classbytes = new byte [(int)f.length()];
			data.readFully(classbytes);
			data.close();
				
			return defineClass(name, classbytes, 0, (int)f.length());
			}
		catch (Exception e) { throw new ClassNotFoundException(name); }
	}

}
