/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package swing.addon.plaf.themelf;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;

public class ThemeLF_InternalFrameUI extends BasicInternalFrameUI {
	private static final boolean DEBUG = false;

	ThemeLF_Resource	resource 			= ThemeLF.getUI("InternalFrame");
	IconUIResource		closeIcon			= resource.getIcon("closeIcon");
	IconUIResource		iconifyIcon			= resource.getIcon("iconifyIcon");
	IconUIResource		resizeIcon			= resource.getIcon("resizeIcon");
	FontUIResource		titleFont			= resource.getFont("titleFont");
	ColorUIResource		titleFontColor		= resource.getColor("titleFontColor");
	ColorUIResource		selectedColor		= resource.getColor("selectedColor");
	ColorUIResource		backgroundColor	= resource.getColor("backgroundColor");
	ThemeLF_Border		titleBorder			= resource.getBorder("titleBorder");
	ThemeLF_Border		iconBorder			= resource.getBorder("iconBorder");
	
	public class TitlePane extends JPanel implements Border, MouseListener, ActionListener {
		JButton				closeButton		= new JButton(closeIcon);
		JButton				iconifyButton	= new JButton(iconifyIcon);
		JButton				resizeButton	= new JButton(resizeIcon);
		JLabel				title				= new JLabel("") {
												public Dimension getPreferredSize() { 
													return new Dimension(Short.MAX_VALUE, Math.max(closeButton.getSize().height, super.getPreferredSize().height)); 
												}};
		
		public TitlePane()  {
			// Layout
			setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			setBorder(this);
			
			// Knpfe initialisieren
			closeButton.setOpaque(false);
			closeButton.setRequestFocusEnabled(false);
			closeButton.setMargin(new Insets(1,1,1,1));
			closeButton.setBorder(BorderFactory.createEmptyBorder(1,1,1,1));
			closeButton.addMouseListener(this);
			
			iconifyButton.setOpaque(false);
			iconifyButton.setRequestFocusEnabled(false);
			iconifyButton.setMargin(new Insets(1,1,1,1));
			iconifyButton.setBorder(BorderFactory.createEmptyBorder(1,1,1,1));
			iconifyButton.addMouseListener(this);
			
			resizeButton.setOpaque(false);
			resizeButton.setRequestFocusEnabled(false);
			resizeButton.setMargin(new Insets(1,1,1,1));
			resizeButton.setBorder(BorderFactory.createEmptyBorder(1,1,1,1));
			resizeButton.addMouseListener(this);
			
			title.setText(frame.getTitle());
			title.setFont(titleFont);
			title.setForeground(titleFontColor);
			
			// Listeners
			closeButton.addActionListener(this);
			iconifyButton.addActionListener(this);
			resizeButton.addActionListener(this);
			
			// hinzufgen
			add(new JStrut(2));
			add(closeButton);
			add(new JStrut(2));
			add(title);
			add(new JStrut(2));
			add(iconifyButton);
			add(new JStrut(2));
			add(resizeButton);
			add(new JStrut(2));
		}

		public void actionPerformed(java.awt.event.ActionEvent arg0)  {
			Object source = arg0.getSource();
			
			if (source == closeButton)
				{
				try { frame.setClosed(true); } catch (PropertyVetoException e0) { }
				}
			else if (source == iconifyButton)
				{
				try { frame.setIcon( ! frame.isIcon()); } catch (PropertyVetoException e1) { }
				}
			else
				{
				try { frame.setMaximum(!frame.isMaximum()); } catch (PropertyVetoException e5) { }
				}
		}

		public java.awt.Insets getBorderInsets(Component c)  {
			return titleBorder.getBorderInsets(c);
		}

		public Dimension getMaximumSize()  {
			return new Dimension(Short.MAX_VALUE, getPreferredSize().height);
		}

		public Dimension getMinimumSize()  {
			return getPreferredSize();
		}

		public Dimension getPreferredSize()  {
			FontMetrics fm = getFontMetrics(titleFont);
			
			int width = 200;
			int height = fm.getAscent() + fm.getDescent() + 4;
			
			return new Dimension(width, height);
		}

		public boolean isBorderOpaque()  {
			return true;
		}

		public void mouseClicked(java.awt.event.MouseEvent arg0)  {
		}

		public void mouseEntered(java.awt.event.MouseEvent arg0)  {
			JButton source = (JButton)arg0.getSource();
			source.setBorder(iconBorder);
		}

		public void mouseExited(java.awt.event.MouseEvent arg0)  {
			JButton source = (JButton)arg0.getSource();
			Insets i = iconBorder.getBorderInsets(null);
			source.setBorder(BorderFactory.createEmptyBorder(i.top, i.left, i.bottom, i.right));
		}

		public void mousePressed(java.awt.event.MouseEvent arg0)  {
		}

		public void mouseReleased(java.awt.event.MouseEvent arg0)  {
			mouseExited(arg0);
		}

		public void paint(Graphics g)  {
			title.setText(frame.getTitle());
			
			closeButton.setVisible(frame.isIconifiable());
			iconifyButton.setVisible(frame.isResizable());
			resizeButton.setVisible(frame.isClosable());
			
			super.paint(g);
		}

		public void paintBorder(Component c, Graphics g, int x, int y, int w, int h)  {
			titleBorder.paintBorder(c, g, x, y, w, h);
		}

		public void paintComponent(Graphics g)  {
			Dimension size = getSize();
			
			// Hintergrund malen
			g.setColor(frame.isSelected() ? selectedColor : backgroundColor);
			g.fillRect(0,0,size.width,size.height);
		}

	}

	public ThemeLF_InternalFrameUI(JInternalFrame b)  {
		super(b);
	}

	protected JComponent createNorthPane(JInternalFrame f)  {
		return new TitlePane();
	}

	public static ComponentUI createUI(JComponent c)  {
		return new ThemeLF_InternalFrameUI((JInternalFrame)c);
	}

	public void installUI(JComponent c)  {
		super.installUI(c);
		
		resource 			= ThemeLF.getUI("InternalFrame");
		closeIcon			= resource.getIcon("closeIcon");
		iconifyIcon			= resource.getIcon("iconifyIcon");
		resizeIcon			= resource.getIcon("resizeIcon");
		titleFont			= resource.getFont("titleFont");
		titleFontColor		= resource.getColor("titleFontColor");
		selectedColor		= resource.getColor("selectedColor");
		backgroundColor	= resource.getColor("backgroundColor");
		titleBorder			= resource.getBorder("titleBorder");
		iconBorder			= resource.getBorder("iconBorder");
	}

}
