/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package swing.addon.plaf.themelf;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.plaf.*;
import java.text.MessageFormat;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

public class ThemeLF_Prefs extends JFrame {
	private static final boolean DEBUG = false;

	// Ressourcen
	final static ResourceBundle resources = ResourceBundle.getBundle("swing.addon.plaf.themelf.resources.themelf");
	
	public 		static final 	String 	DEFAULT_THEME	= "Thin3D";
	private 	static final 	File 	prefsDir				= new File(System.getProperty("user.home"), ".themelf");
	private 	static		String	style 				= DEFAULT_THEME;
	
	// Komponenten
	JButton 		save 						= new JButton(GRS("PrefsFrame.Buttons.Save"));
	JButton 		test							= new JButton(GRS("PrefsFrame.Buttons.Test"));
	JButton 		cancel						= new JButton(GRS("PrefsFrame.Buttons.Cancel"));
	
	// Themes Panel
	JList			themes						= new JList();
	JCheckBox		keep						= new JCheckBox(GRS("PrefsFrame.Themes.Keep"), true);
	
	// Font Panel
	JFontSelector	normalFont					= new JFontSelector();
	JFontSelector	smallFont					= new JFontSelector();
	
	// Color Panel
	JColorSelector	backgroundColor				= new JColorSelector();
	JColorSelector	caretColor					= new JColorSelector();
	JColorSelector	focusColor					= new JColorSelector();
	JColorSelector	selectionColor				= new JColorSelector();
	JColorSelector	textColor					= new JColorSelector();
	JColorSelector	lightEdgeColor				= new JColorSelector();
	JColorSelector	darkEdgeColor				= new JColorSelector();
	
	public ThemeLF_Prefs()  {
		super(GRS("PrefsFrame.Title"));
		
		// einrichten
		if (DEBUG) System.out.println("ThemeLF_Prefs(): found " + ThemeLF.getThemes().length + " different themes");
		themes.setListData(ThemeLF.getThemes());
		
		// Panel erzeugen
		JPanel p =(JPanel) getContentPane();
		p.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
		p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
		p.add(createTabPanel());
		p.add(new Strut());
		p.add(new JPopupMenu.Separator());
		p.add(new Strut());
		p.add(createButtons());
		
		// WindowListener
		addWindowListener(new WindowAdapter() { public void windowClosing(WindowEvent e) { System.exit(0); }});
		
		// Menus
		setJMenuBar(createMenuBar());
		
		// Komponenten einrichten
		updateComponents();
		
		// Anzeigen
		setLocation(100,100);
		setVisible(true);
		pack();
	}

	static {
		prefsDir.mkdirs();
	}

	static String GRS(String id)  {
		try
			{
			return resources.getString(id);
			}
		catch (MissingResourceException x) 
			{
			System.err.println(x.getLocalizedMessage() + ": '" + id + "'" );
			return "MISSING RESOURCE: " + id;
			}
	}

	static String GRS(String id, String arg1)  {
		String res = GRS(id);
		
		try
			{
			res = MessageFormat.format(res, new Object [] { arg1 });
			}
		catch (Exception x) { System.err.println(x); }
		
		return res;
	}

	private void about()  {
		JOptionPane.showMessageDialog(this, UIManager.getLookAndFeel().getDescription());
	}

	private JPanel createButtons()  {
		JPanel result = new JPanel();
		
		result.setLayout(new BoxLayout(result, BoxLayout.X_AXIS));
		result.add(save);
		result.add(Box.createHorizontalGlue());
		result.add(test);
		result.add(Box.createHorizontalGlue());
		result.add(cancel);
		
		save.addActionListener(new ActionListener() { 
			public void actionPerformed(ActionEvent e) { 
				test();
				saveUserData();
				System.exit(0);
			}
		});
		
		test.addActionListener(new ActionListener() { 
			public void actionPerformed(ActionEvent e) { 
				test();
			}
		});
		
		cancel.addActionListener(new ActionListener() { 
			public void actionPerformed(ActionEvent e) { 
				System.exit(0);
			}
		});
		
		return result;
	}

	private JMenuBar createMenuBar()  {
		JMenuBar 	result 	= new JMenuBar();
		JMenu 		menu	= null;
		JMenuItem	item	= null;
		
		// Program
		menu = result.add(new JMenu(GRS("PrefsFrame.Menu.Program")));
		item = menu.add(new JMenuItem(GRS("PrefsFrame.Menu.About")));
		item.setAccelerator(KeyStroke.getKeyStroke("control A"));
		item.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { about(); }});
		menu.addSeparator();
		item = menu.add(new JMenuItem(GRS("PrefsFrame.Menu.Quit")));
		item.setAccelerator(KeyStroke.getKeyStroke("control Q"));
		item.addActionListener(new ActionListener() { public void actionPerformed(ActionEvent e) { System.exit(0); }});
		
		return result;
	}

	private JPanel createPanel_Colors()  {
		JPanel result = new JPanel();
		result.setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(4,4,4,4);
		
		c.gridx=0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel(GRS("PrefsFrame.Colors.Label1")), c);
		
		c.gridx=1; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(backgroundColor, c);
		
		c.gridx=0; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel(GRS("PrefsFrame.Colors.Label2")), c);
		
		c.gridx=1; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(caretColor, c);
		
		c.gridx=0; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel(GRS("PrefsFrame.Colors.Label3")), c);
		
		c.gridx=1; c.gridy=2; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(focusColor, c);
		
		c.gridx=0; c.gridy=3; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel(GRS("PrefsFrame.Colors.Label4")), c);
		
		c.gridx=1; c.gridy=3; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(selectionColor, c);
		
		c.gridx=0; c.gridy=4; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel(GRS("PrefsFrame.Colors.Label5")), c);
		
		c.gridx=1; c.gridy=4; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(textColor, c);
		
		c.gridx=2; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel(GRS("PrefsFrame.Colors.Label6")), c);
		
		c.gridx=3; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(lightEdgeColor, c);
		
		c.gridx=2; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel(GRS("PrefsFrame.Colors.Label7")), c);
		
		c.gridx=3; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(darkEdgeColor, c);
		
		return result;
	}

	private JPanel createPanel_Fonts()  {
		JPanel result = new JPanel();
		result.setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(4,4,4,4);
		
		c.gridx=0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel(GRS("PrefsFrame.Fonts.Label1")), c);
		
		c.gridx=1; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(normalFont, c);
		
		c.gridx=0; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(new JLabel(GRS("PrefsFrame.Fonts.Label2")), c);
		
		c.gridx=1; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=0; c.weighty=0;
		result.add(smallFont, c);
		
		return result;
	}

	private JPanel createPanel_Themes()  {
		JPanel result = new JPanel();
		result.setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(4,4,4,4);
		
		c.gridx=0; c.gridy=0; c.gridwidth=1; c.gridheight=1; c.weightx=1; c.weighty=1;
		result.add(new JScrollPane(themes, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED), c);
		
		c.gridx=0; c.gridy=1; c.gridwidth=1; c.gridheight=1; c.weightx=1; c.weighty=0;
		result.add(keep, c);
		
		return result;
	}

	private JTabbedPane createTabPanel()  {
		JTabbedPane result = new JTabbedPane();
		
		result.addTab(GRS("PrefsFrame.TabbedPane.Fonts"), createPanel_Fonts());
		result.addTab(GRS("PrefsFrame.TabbedPane.Colors"), createPanel_Colors());
		result.addTab(GRS("PrefsFrame.TabbedPane.Themes"), createPanel_Themes());
		
		return result;
	}

	public static final ColorUIResource getDefaultColor(String s)  {
		ThemeLF_Resource resource = ThemeLF.getUI("Defaults");
		if (resource == null)  throw new IllegalArgumentException("ThemeLF_Utils.parseColors(): cannot find Defaults file");
		return resource.getColor(s);
	}

	public static final FontUIResource getDefaultFont(String s)  {
		ThemeLF_Resource resource = ThemeLF.getUI("Defaults");
		if (resource == null)  throw new IllegalArgumentException("ThemeLF_Utils.parseColors(): cannot find Defaults file");
		return resource.getFont(s);
	}

	public static final String getStyle()  {
		return style;
	}

	static final void loadUserData()  {
		if (DEBUG) System.out.println("ThemeLF_Prefs.loadUserData()");
		
		try
			{
			File file = new File(prefsDir, "Style");
			BufferedReader in = new BufferedReader(new FileReader(file));
			style = in.readLine();
			in.close();
			}
		catch (FileNotFoundException x1) {  }
		catch (Exception x2) { ThemeLF_Utils.showException(x2); }
		
		try
			{
			File file = new File(prefsDir, "Defaults");
			BufferedReader in = new BufferedReader(new FileReader(file));
			ThemeLF_Resource resource = ThemeLF.getUI("Defaults");
			resource.readFrom(in);
			in.close();
			}
		catch (FileNotFoundException x1) {  }
		catch (Exception x2) { ThemeLF_Utils.showException(x2); }
	}

	public static void main(String [] args)  {
		try 
			{
			UIManager.installLookAndFeel("Theme Look & Feel", "swing.addon.plaf.thememeLF");
			UIManager.setLookAndFeel("swing.addon.plaf.themelf.ThemeLF");
			} 
		catch (Exception exc) 
			{
			System.err.println("Error loading L&F: " + exc);
			}
		
		new ThemeLF_Prefs();
	}

	static final void saveUserData()  {
		if (DEBUG) System.out.println("ThemeLF_Prefs.saveUserData()");
		
		try
			{
			File file = new File(prefsDir, "Style");
			PrintWriter out = new PrintWriter(new FileWriter(file));
			out.println(style);
			out.close();
			}
		catch (Exception x) { ThemeLF_Utils.showException(x); }
		
		try
			{
			File file = new File(prefsDir, "Defaults");
			PrintWriter out = new PrintWriter(new FileWriter(file));
			ThemeLF_Resource resource = ThemeLF.getUI("Defaults");
			resource.writeTo(out);
			out.close();
			}
		catch (Exception x) { ThemeLF_Utils.showException(x); }
	}

	public static final void setDefaultColor(String s, ColorUIResource color)  {
		ThemeLF_Resource resource = ThemeLF.getUI("Defaults");
		if (resource == null)  throw new IllegalArgumentException("ThemeLF_Utils.parseColors(): cannot find Defaults file");
		resource.setColor(s, color);
	}

	public static final void setDefaultFont(String s, FontUIResource font)  {
		ThemeLF_Resource resource = ThemeLF.getUI("Defaults");
		if (resource == null)  throw new IllegalArgumentException("ThemeLF_Utils.parseColors(): cannot find Defaults file");
		resource.setFont(s, font);
	}

	private void test()  {
		FontUIResource f1,f2;
		ColorUIResource c1,c2,c3,c4,c5,c6,c7;
		
		// Fonts
		setDefaultFont("normalFont", f1 = normalFont.getFontUIResource());
		setDefaultFont("smallFont", f2 = smallFont.getFontUIResource());
		
		// Colors
		setDefaultColor("backgroundColor", c1 = backgroundColor.getColorUIResource());
		setDefaultColor("caretColor", c2 = caretColor.getColorUIResource());
		setDefaultColor("focusColor", c3 = focusColor.getColorUIResource());
		setDefaultColor("selectionColor", c4 = selectionColor.getColorUIResource());
		setDefaultColor("textColor", c5 = textColor.getColorUIResource());
		setDefaultColor("lightEdgeColor", c6 = lightEdgeColor.getColorUIResource());
		setDefaultColor("darkEdgeColor", c7 = darkEdgeColor.getColorUIResource());
		
		// Themes
		style = (String) themes.getSelectedValue();
		
		// Setzen
		updateLookAndFeel();
		updateComponents();
		
		// Falls die alten Font und Color-Werte bleiben sollen
		if (keep.isSelected())
			{
			setDefaultFont("normalFont", f1);
			setDefaultFont("smallFont", f2);
			setDefaultColor("backgroundColor", c1);
			setDefaultColor("caretColor", c2);
			setDefaultColor("focusColor", c3);
			setDefaultColor("selectionColor", c4);
			setDefaultColor("textColor", c5);
			setDefaultColor("lightEdgeColor", c6);
			setDefaultColor("darkEdgeColor", c7);
			updateLookAndFeel();
			updateComponents();
			}
	}

	private void updateComponents()  {
		// Font Panel
		normalFont.setUserFont(ThemeLF_Prefs.getDefaultFont("normalFont"));
		smallFont.setUserFont(ThemeLF_Prefs.getDefaultFont("smallFont"));
		
		// Color Panel
		backgroundColor.setColor(ThemeLF_Prefs.getDefaultColor("backgroundColor"));
		caretColor.setColor(ThemeLF_Prefs.getDefaultColor("caretColor"));
		focusColor.setColor(ThemeLF_Prefs.getDefaultColor("focusColor"));
		selectionColor.setColor(ThemeLF_Prefs.getDefaultColor("selectionColor"));
		textColor.setColor(ThemeLF_Prefs.getDefaultColor("textColor"));
		lightEdgeColor.setColor(ThemeLF_Prefs.getDefaultColor("lightEdgeColor"));
		darkEdgeColor.setColor(ThemeLF_Prefs.getDefaultColor("darkEdgeColor"));
		
		// Themes Panel
		themes.setSelectedValue(style, true);
	}

	private void updateLookAndFeel()  {
		if (DEBUG) System.out.println("ThemeLF_Prefs.updateLookAndFeel()");
		
		try 
			{
			UIManager.setLookAndFeel("swing.addon.plaf.themelf.ThemeLF");
			SwingUtilities.updateComponentTreeUI(SwingUtilities.getRoot(this));
			Dimension preferred = getPreferredSize();
			Dimension size = getSize();
			if (preferred.width > size.width) size.width = preferred.width;
			if (preferred.height > size.height) size.height = preferred.height;
			setSize(size);
			} 
		catch (Exception exc) { System.err.println(exc); }
	}

}
