/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package swing.addon.plaf.themelf;

import java.awt.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.plaf.*;

public final class ThemeLF_Resource {
	private static final boolean DEBUG = false;

	File basedir = new File(ThemeLF.getInstallationDirectory(), "data" + File.separator + "ui");
	String type = null;
	String name = null;
	Hashtable data = new Hashtable();
	
	public ThemeLF_Resource(String style, String id)  {
		name= id;
		type = style;
		loadResourceData(style, id);
	}

	public boolean getBoolean(String id)  {
		String s = (String) data.get(id);
		if (s == null) 
			{
			System.err.println("Can't find property '"+id+"' in '" + name+"', using default value 'false'.");
			return false;
			}
		
		return s.equals("true");
	}

	public ThemeLF_Border getBorder(String id)  {
		Object o = data.get(id);
		if (o == null) 
			{
			System.err.println("Can't find property '"+id+"' in '" + name+"', using default value 'null'.");
			return new ThemeLF_Border("null");
			}
		
		// Schon umgewandelt
		if (o instanceof ThemeLF_Border) return (ThemeLF_Border)o;
		
		// Noch nicht umgewandelt, wird nicht gespeichert wegen Prefs-Programm und updateLookAndFeel()
		return new ThemeLF_Border((String)o);
	}

	public ColorUIResource getColor(String id)  {
		Object o = data.get(id);
		if (o == null) 
			{
			System.err.println("Can't find property '"+id+"' in '" + name+"', using default value '255,0,0'.");
			return new ColorUIResource(255,0,0);
			}
		
		// Schon umgewandelt
		if (o instanceof ColorUIResource) return (ColorUIResource)o;
		
		// Noch nicht umgewandelt, wird nicht gespeichert wegen Prefs-Programm und updateLookAndFeel()
		return ThemeLF_Utils.parseColor((String)o);
	}

	public Dimension getDimension(String id)  {
		Object o = data.get(id);
		if (o == null) 
			{
			System.err.println("Can't find property '"+id+"' in '" + name+"', using default value '5,5'.");
			return new DimensionUIResource(5,5);
			}
		
		// Schon umgewandelt
		if (o instanceof Dimension) return (Dimension)o;
		
		// Noch nicht umgewandelt
		Dimension result = ThemeLF_Utils.parseDimension((String)o);
		data.put(id, result);
		return result;
	}

	public ThemeLF_Disabler getDisabler(String id)  {
		Object o = data.get(id);
		if (o == null) 
			{
			System.err.println("Can't find property '"+id+"' in '" + name+"', using default value 'null'.");
			return new ThemeLF_Disabler("null");
			}
		
		// Schon umgewandelt
		if (o instanceof ThemeLF_Disabler) return (ThemeLF_Disabler)o;
		
		// Noch nicht umgewandelt, wird nicht gespeichert wegen Prefs-Programm und updateLookAndFeel()
		return ThemeLF_Utils.parseDisabler((String)o);
	}

	public FontUIResource getFont(String id)  {
		Object o = data.get(id);
		if (o == null) 
			{
			System.err.println("Can't find property '"+id+"' in '" + name+"', using default value 'Verdana,12'.");
			return new FontUIResource("Verdana", Font.PLAIN, 12);
			}
		
		// Schon umgewandelt
		if (o instanceof FontUIResource) return (FontUIResource)o;
		
		// Noch nicht umgewandelt, wird nicht gespeichert wegen Prefs-Programm und updateLookAndFeel()
		return ThemeLF_Utils.parseFont((String)o);
	}

	public IconUIResource getIcon(String id)  {
		Object o = data.get(id);
		if (o == null) 
			{
			System.err.println("Can't find property '"+id+"' in '" + name+"', cannot continue program'.");
			System.exit(10);
			}
		
		// Schon umgewandelt
		if (o instanceof IconUIResource) return (IconUIResource)o;
		
		// Noch nicht umgewandelt
		IconUIResource result = ThemeLF_Utils.loadIcon((String)o);
		data.put(id, result);
		return result;
	}

	public InsetsUIResource getInsets(String id)  {
		Object o = data.get(id);
		if (o == null) 
			{
			System.err.println("Can't find property '"+id+"' in '" + name+"', using default value '5,5,5,5'.");
			return new InsetsUIResource(5,5,5,5);
			}
		
		// Schon umgewandelt
		if (o instanceof InsetsUIResource) return (InsetsUIResource)o;
		
		// Noch nicht umgewandelt
		InsetsUIResource result = ThemeLF_Utils.parseInsets((String)o);
		data.put(id, result);
		return result;
	}

	public int getInteger(String id)  {
		String s = (String) data.get(id);
		if (s == null) 
			{
			System.err.println("Can't find property '"+id+"' in '" + name+"', using default value 'Integer.MAX_VALUE'.");
			return Integer.MAX_VALUE;
			}
		
		return Integer.parseInt(s);
	}

	public void loadResourceData(String style, String file)  {
		String baseStyle = null;
		
		try
			{
			File f = new File(basedir, style + File.separator + "Extends");
			BufferedReader in = new BufferedReader(new FileReader(f));
			baseStyle = in.readLine();
			if (DEBUG) System.out.println("ThemeLF_Resource.loadResourceData(): " + style + "/" + file + " extends " + baseStyle + "/" + file);
			loadResourceData(baseStyle, file);
			in.close();
			}
		catch (FileNotFoundException x1)  { }
		catch (Exception x)  { throw new ThemeLF_Exception("Error while reading '" + file + "'", x); }
		
		try
			{
			File f = new File(basedir, style + File.separator + file);
			BufferedReader in = new BufferedReader(new FileReader(f));
			readFrom(in);
			in.close();
			}
		catch (FileNotFoundException x1)  { if (baseStyle == null) throw new ThemeLF_Exception("Error while reading '" + file + "'", x1); }
		catch (Exception x) { throw new ThemeLF_Exception("Error while reading '" + file + "'", x); }
	}

	void readFrom(BufferedReader in) throws IOException {
		while (true)
			{
			String line = in.readLine();
			if (line == null) break;
			line = line.trim();
			if (line.startsWith("#")) continue;
			if (line.length()==0) continue;
			StringTokenizer tok = new StringTokenizer(line, "=");
			String tag = tok.nextToken();
			String value = tok.hasMoreElements() ? tok.nextToken() : "";
			data.put(tag, value);
			}
	}

	public void setColor(String id, ColorUIResource color)  {
		data.put(id, color);
	}

	public void setFont(String id, FontUIResource font)  {
		data.put(id, font);
	}

	public String toString()  {
		return getClass().getName() + "[" + data.toString() + "]";
	}

	void writeTo(PrintWriter out) throws IOException {
		for (Enumeration e = data.keys(); e.hasMoreElements(); )
			{
			String key = (String) e.nextElement();
			Object value = data.get(key);
			if (value instanceof String)
				{
				out.println(key + "=" + value);
				}
			else if (value instanceof ColorUIResource)
				{
				ColorUIResource c = (ColorUIResource) value;
				out.println(key + "=" + c.getRed() + "," + c.getGreen() + "," + c.getBlue());
				}
			else if (value instanceof FontUIResource)
				{
				FontUIResource f = (FontUIResource) value;
				out.println(key + "=" + f.getName() + "," + f.getSize());
				}
			else ThemeLF_Utils.showError("Cannot write " + value + " to prefs file.");
			}
	}

}
