/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package swing.addon.plaf.themelf;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;

public class ThemeLF_ScrollBarUI extends BasicScrollBarUI {
	private static final boolean DEBUG = false;

	JScrollBar			scrollbar						= null;
	ThemeLF_Resource resource 						= null;
	// ScrollBar
	ThemeLF_Border	thumbBorder					= null;
	ThemeLF_Border	trackBorder						= null;
	ColorUIResource	thumbColor						= null;
	ColorUIResource	trackColor						= null;
	IconUIResource	horizontalThumbPattern		= null;
	IconUIResource	verticalThumbPattern			= null;
	boolean			centerHorizontalThumbPattern	= false;
	boolean			centerVerticalThumbPattern	= false;
	Insets 				trackBorderInsets 				= null;
	// Button
	ThemeLF_Border 	normalBorder 					= null;
	ThemeLF_Border 	pressedBorder 					= null;
	ColorUIResource	backgroundColor				= null;
	ColorUIResource	pressedBackgroundColor		= null;
	IconUIResource	arrowLeft						= null;
	IconUIResource	arrowRight						= null;
	IconUIResource	arrowUp						= null;
	IconUIResource	arrowDown						= null;
	
	public class Button extends BasicArrowButton {
		int orientation;
		
		public Button(int orientation)  {
			super(orientation);
			
			this.orientation = orientation;
		}

		public Dimension getMinimumSize()  {
			return getPreferredSize();
		}

		public Dimension getPreferredSize()  {
			int w, h;
			Insets borderInsets = normalBorder.getBorderInsets(this);
			
			switch (orientation)
				{
				case NORTH:
					w = arrowUp.getIconWidth();
					h = arrowUp.getIconHeight();
				case SOUTH:
					w = arrowDown.getIconWidth();
					h = arrowDown.getIconHeight();
				case EAST:
					w = arrowRight.getIconWidth();
					h = arrowRight.getIconHeight();
				default:
					w = arrowLeft.getIconWidth();
					h = arrowLeft.getIconHeight();
				}
			
			return new Dimension(w+4+borderInsets.left+borderInsets.right, h+4+borderInsets.top+borderInsets.bottom);
		}

		public void paint(Graphics g)  {
			Dimension size = getSize();
			
			g.setColor(backgroundColor);
			g.fillRect(0, 0, size.width, size.height);
			
			if (getModel().isPressed())
				{
				g.setColor(pressedBackgroundColor);
				g.fillRect(1, 1, size.width-2, size.height-2);
				pressedBorder.paintBorder(this, g, 0, 0, size.width, size.height);
				}
			else normalBorder.paintBorder(this, g, 0, 0, size.width, size.height);
			
			switch (orientation)
				{
				case NORTH:
					arrowUp.paintIcon(this, g, (size.width - arrowUp.getIconWidth())/2, (size.height - arrowUp.getIconHeight())/2);
					break;
				case SOUTH:
					arrowDown.paintIcon(this, g, (size.width - arrowDown.getIconWidth())/2, (size.height - arrowDown.getIconHeight())/2);
					break;
				case EAST:
					arrowRight.paintIcon(this, g, (size.width - arrowRight.getIconWidth())/2, (size.height - arrowRight.getIconHeight())/2);
					break;
				case WEST:
					arrowLeft.paintIcon(this, g, (size.width - arrowLeft.getIconWidth())/2, (size.height - arrowLeft.getIconHeight())/2);
					break;
				}
		}

	}

	public ThemeLF_ScrollBarUI(JScrollBar bar)  {
		scrollbar = bar;
	}

	protected JButton createDecreaseButton(int orientation)  {
		return new ThemeLF_ScrollBarUI.Button(orientation);
	}

	protected JButton createIncreaseButton(int orientation)  {
		return new ThemeLF_ScrollBarUI.Button(orientation);
	}

	public static ComponentUI createUI(JComponent c)  {
		return new ThemeLF_ScrollBarUI((JScrollBar)c);
	}

	public void installUI(JComponent c)  {
		super.installUI(c);
		
		resource 						= ThemeLF.getUI("ScrollBar");
		// ScrollBar
		trackColor						= resource.getColor("trackColor");
		trackBorder					= resource.getBorder("trackBorder");
		thumbColor					= resource.getColor("thumbColor");
		thumbBorder					= resource.getBorder("thumbBorder");
		horizontalThumbPattern		= resource.getIcon("horizontalThumbPattern");
		verticalThumbPattern			= resource.getIcon("verticalThumbPattern");
		centerHorizontalThumbPattern	= resource.getBoolean("centerHorizontalThumbPattern");
		centerVerticalThumbPattern	= resource.getBoolean("centerVerticalThumbPattern");
		trackBorderInsets 				= trackBorder.getBorderInsets(scrollbar);
		// Button
		normalBorder 					= resource.getBorder("button.normalBorder");
		pressedBorder 					= resource.getBorder("button.pressedBorder");
		backgroundColor				= resource.getColor("button.normalBackgroundColor");
		pressedBackgroundColor		= resource.getColor("button.pressedBackgroundColor");
		arrowLeft						= resource.getIcon("button.arrowLeft");
		arrowRight						= resource.getIcon("button.arrowRight");
		arrowUp						= resource.getIcon("button.arrowUp");
		arrowDown					= resource.getIcon("button.arrowDown");
		
		if (horizontalThumbPattern.getIconWidth() == -1) horizontalThumbPattern=null;
		if (verticalThumbPattern.getIconWidth() == -1) verticalThumbPattern=null;
	}

	protected void paintThumb(Graphics g, JComponent c, Rectangle thumbBounds)  {
		Border 	border 			= thumbBorder;
		Color 	color 			= thumbColor;
		Insets 	borderInsets 	= thumbBorder.getBorderInsets(c);
		
		Rectangle draw = (Rectangle) thumbBounds.clone();
		
		// Falls nichts zu scrollen da ist
		if (thumbBounds.x == 0 && thumbBounds.y == 0 && thumbBounds.width == 0 && thumbBounds.height == 0) draw = (Rectangle)trackRect.clone();
		
		// Den ueren Rahmen abziehen
		draw.x += trackBorderInsets.left;
		draw.y += trackBorderInsets.top;
		draw.width -= trackBorderInsets.left+trackBorderInsets.right;
		draw.height -= trackBorderInsets.top+trackBorderInsets.bottom;
		
		// Rahmen des Thumb zeichnen
		border.paintBorder(scrollbar,  g, draw.x, draw.y, draw.width, draw.height);
		draw.x += borderInsets.left;
		draw.y += borderInsets.top;
		draw.width -= borderInsets.left + borderInsets.right;
		draw.height -= borderInsets.top + borderInsets.bottom;
		
		// Ausmalen
		g.setColor(color);
		g.fillRect(draw.x, draw.y, draw.width, draw.height);
		
		
		// Pattern
		Insets insets = border.getBorderInsets(scrollbar);
		Shape clip = g.getClip();
		g.setClip(new Rectangle(draw.x+1, draw.y+1, draw.width-2, draw.height-2));
		
		if (scrollbar.getOrientation() == JScrollBar.HORIZONTAL && horizontalThumbPattern != null)
			{
			if (centerHorizontalThumbPattern)
				{
				int x = draw.x + ((draw.width - horizontalThumbPattern.getIconWidth()) / 2);
				int y = draw.y + ((draw.height - horizontalThumbPattern.getIconHeight()) / 2);
				horizontalThumbPattern.paintIcon(scrollbar, g, x, y);
				}
			else
				{
				for (int y=draw.y; y < draw.y+draw.height; y+=horizontalThumbPattern.getIconHeight())
					for (int x=draw.x; x<draw.x+draw.width; x+=horizontalThumbPattern.getIconWidth())
						horizontalThumbPattern.paintIcon(scrollbar, g, x, y);
				}
			}
		else if (scrollbar.getOrientation() == JScrollBar.VERTICAL && verticalThumbPattern != null)
			{
			if (centerVerticalThumbPattern)
				{
				int x = draw.x + ((draw.width - verticalThumbPattern.getIconWidth()) / 2);
				int y = draw.y + ((draw.height - verticalThumbPattern.getIconHeight()) / 2);
				verticalThumbPattern.paintIcon(scrollbar, g, x, y);
				}
			else
				{
				for (int y=draw.y; y < draw.y+draw.height; y+=verticalThumbPattern.getIconHeight())
					for (int x=draw.x; x<draw.x+draw.width; x+=verticalThumbPattern.getIconWidth())
						verticalThumbPattern.paintIcon(scrollbar, g, x, y);
				}
			}
		
		g.setClip(clip);
	}

	protected void paintTrack(Graphics g, JComponent c, Rectangle trackBounds)  {
		// Vertiefung ausmalen
		g.setColor(scrollbar.getBackground());
		g.fillRect(0,0,c.getWidth(), c.getHeight());
		
		// Vertiefung einfrben
		Insets i = trackBorder.getBorderInsets(scrollbar);
		g.setColor(trackColor);
		g.fillRect(trackBounds.x+i.left,trackBounds.y+i.top,trackBounds.width-i.left-i.right,trackBounds.height-i.top-i.bottom);
		
		// Rahmen zeichnen
		trackBorder.paintBorder(scrollbar, g, trackBounds.x, trackBounds.y, trackBounds.width, trackBounds.height);
	}

}
