/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package swing.addon.plaf.themelf;

import java.awt.*;
import javax.swing.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;

public class ThemeLF_SeparatorUI extends BasicSeparatorUI {
	private static final boolean DEBUG = false;

	ThemeLF_Resource 	resource 		= null;
	ColorUIResource 	lightEdgeColor 	= null;
	ColorUIResource 	darkEdgeColor 	= null;
	
	public static ComponentUI createUI(JComponent c)  {
		return new ThemeLF_SeparatorUI();
	}

	public Dimension getMaximumSize(JComponent c)  {
		if (((JSeparator)c).getOrientation() == JSeparator.VERTICAL)
			{
			return new Dimension(2, Short.MAX_VALUE);
			}
		else
			{
			return new Dimension(Short.MAX_VALUE, 2);
			}
	}

	public Dimension getPreferredSize(JComponent c)  {
		if (((JSeparator)c).getOrientation() == JSeparator.VERTICAL)
			{
			return new Dimension( 2, 0 );
			}
		else
			{
			return new Dimension( 0, 2 );
			}
	}

	public void installUI(JComponent c)  {
		super.installUI(c);
		
		resource		= ThemeLF.getUI("Separator");
		lightEdgeColor 	= resource.getColor("lightEdgeColor");
		darkEdgeColor	= resource.getColor("darkEdgeColor");
	}

	public void paint(Graphics g, JComponent c)  {
		Dimension s = c.getSize();
		
		if ( ((JSeparator)c).getOrientation() == JSeparator.VERTICAL )
			{
			g.setColor(lightEdgeColor);
			g.drawLine( 0, 0, 0, s.height );
			g.setColor(darkEdgeColor);
			g.drawLine( 1, 0, 1, s.height );
			}
		else  // HORIZONTAL
			{
			g.setColor(lightEdgeColor);
			g.drawLine( 0, 0, s.width, 0 );
			g.setColor(darkEdgeColor);
			g.drawLine( 0, 1, s.width, 1 );
			}
	}

}
