/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package swing.addon.plaf.themelf;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;

public class ThemeLF_SliderUI extends BasicSliderUI {
	private static final boolean DEBUG = false;

	ThemeLF_Resource	resource 					= ThemeLF.getUI("Slider");
	ColorUIResource		focusColor					= null;
	ColorUIResource		thumbBackgroundColor		= null;
	ColorUIResource		trackBackgroundColor		= null;
	IconUIResource		thumbIconHorizontal		= null;
	Insets					thumbIconHorizontalInsets	= null;
	IconUIResource		thumbIconVertical			= null;
	Insets					thumbIconVerticalInsets	= null;
	Insets					thumbBorderInsets			= null;
	ThemeLF_Border		thumbBorder				= null;
	ThemeLF_Border		trackBorder					= null;
	ThemeLF_Disabler 	disabler 					= null;
	
	public ThemeLF_SliderUI(JSlider s)  {
		super(s);
	}

	public static ComponentUI createUI(JComponent c)  {
		return new ThemeLF_SliderUI((JSlider)c);
	}

	protected Dimension getThumbSize()  {
		if (slider.getOrientation() == JSlider.HORIZONTAL)
			{
			if (thumbIconHorizontal == null) return new Dimension(11,20);
			return new Dimension(
				thumbIconHorizontal.getIconWidth() + thumbBorderInsets.left + thumbBorderInsets.right + thumbIconHorizontalInsets.left + thumbIconHorizontalInsets.right, 
				thumbIconHorizontal.getIconHeight() + thumbBorderInsets.top + thumbBorderInsets.bottom + thumbIconHorizontalInsets.top + thumbIconVerticalInsets.bottom);
			}
		else
			{
			if (thumbIconVertical == null) return new Dimension(20,11); 
			return new Dimension(
				thumbIconVertical.getIconWidth() + thumbBorderInsets.left + thumbBorderInsets.right + thumbIconVerticalInsets.left + thumbIconVerticalInsets.right, 
				thumbIconVertical.getIconHeight() + thumbBorderInsets.top + thumbBorderInsets.bottom + thumbIconVerticalInsets.top + thumbIconVerticalInsets.bottom);
			}
	}

	public void installUI(JComponent c)  {
		super.installUI(c);
		
		resource 					= ThemeLF.getUI("Slider");
		focusColor					= resource.getColor("focusColor");
		thumbBackgroundColor		= resource.getColor("thumbBackgroundColor");
		thumbBorder				= resource.getBorder("thumbBorder");
		thumbIconHorizontal		= resource.getIcon("thumbIconHorizontal");
		thumbIconHorizontalInsets	= resource.getInsets("thumbIconHorizontalInsets");
		thumbIconVertical			= resource.getIcon("thumbIconVertical");
		thumbIconVerticalInsets	= resource.getInsets("thumbIconVerticalInsets");
		trackBackgroundColor		= resource.getColor("trackBackgroundColor");
		trackBorder				= resource.getBorder("trackBorder");
		thumbBorderInsets			= thumbBorder.getBorderInsets(slider);
		disabler 					= resource.getDisabler("disable");
		
		if (thumbIconHorizontal.getIconWidth() == -1) thumbIconHorizontal = null;
		if (thumbIconVertical.getIconWidth() == -1) thumbIconVertical = null;
	}

	public void paint(Graphics g, JComponent c)  {
		super.paint(g, c);
		
		if (! c.isEnabled()) disabler.paint(c, g, 0, 0, 0, 0);
	}

	public void paintFocus(Graphics g)  {
		Dimension size = slider.getSize();
		g.setColor(focusColor);
		g.drawRect(0,0,size.width-1,size.height-1);
	}

	public void paintThumb(Graphics g)  {
		// Hintergrund malen
		g.setColor(slider.getBackground());
		g.fillRect(thumbRect.x, thumbRect.y, thumbRect.width, thumbRect.height);
		
		// Rahmen malen
		thumbBorder.paintBorder(slider, g, thumbRect.x, thumbRect.y, thumbRect.width, thumbRect.height);
		
		// farbig ausmalen
		g.setColor(thumbBackgroundColor);
		g.fillRect(
			thumbRect.x+thumbBorderInsets.left, 
			thumbRect.y+thumbBorderInsets.top, 
			thumbRect.width-thumbBorderInsets.left-thumbBorderInsets.right, 
			thumbRect.height-thumbBorderInsets.top-thumbBorderInsets.bottom);
		
		// Icon malen
		if (slider.getOrientation() == JSlider.HORIZONTAL)
			{
			if (thumbIconHorizontal != null) thumbIconHorizontal.paintIcon(slider, g, 
				thumbRect.x+thumbBorderInsets.left+thumbIconHorizontalInsets.left, 
				thumbRect.y+thumbBorderInsets.top+thumbIconHorizontalInsets.top);
			}
		else
			{
			if (thumbIconVertical != null) thumbIconVertical.paintIcon(slider, g, 
				thumbRect.x+thumbBorderInsets.left+thumbIconVerticalInsets.left, 
				thumbRect.y+thumbBorderInsets.top+thumbIconVerticalInsets.top);
			}
	}

	public void paintTrack(Graphics g)  {
		// Hintergrund lschen
		g.setColor(slider.getBackground());
		g.fillRect(trackRect.x, trackRect.y, trackRect.width-1, trackRect.height-1);
		
		// Insets holen
		Insets i = trackBorder.getBorderInsets(slider);
		
		// berechnen
		Rectangle draw = new Rectangle(trackRect.x, trackRect.y, trackRect.width, trackRect.height);
		if (slider.getOrientation() == JSlider.HORIZONTAL)
			{
			draw.height = trackRect.height/3 + trackRect.height % 3 + i.top + i.bottom;
			draw.y = draw.y + (trackRect.height - draw.height)/2;
			}
		else
			{
			draw.width = trackRect.width/3 + trackRect.width % 3 + i.left + i.right;
			draw.x = draw.x + (trackRect.width - draw.width)/2;
			}
		
		// Rahmen malen
		trackBorder.paintBorder(slider, g, draw.x, draw.y, draw.width, draw.height);
		
		// Vertiefung einfrben
		g.setColor(trackBackgroundColor);
		g.fillRect(draw.x+i.left, draw.y+i.top, draw.width-i.left-i.right, draw.height-i.top-i.bottom);
	}

}
