/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package swing.addon.plaf.themelf;

import java.awt.*;
import javax.swing.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;
import javax.swing.border.*;
import java.awt.event.*;
import java.util.*;

public class ThemeLF_ToolBarUI extends BasicToolBarUI {
	private static final boolean DEBUG = false;

	ThemeLF_Resource	resource 			= null;
	ThemeLF_Disabler 		disabler 			= null;
	
	static Border			activeBorder		= new ThemeLF_ButtonUI_Border();
	static Insets 			insets 				= activeBorder.getBorderInsets(null);
	static Border 			inactiveBorder 		= BorderFactory.createEmptyBorder(insets.top, insets.left, insets.bottom, insets.right);
	static Hashtable 		originalBorders 		= new Hashtable();
	
	static MouseListener 	mouseListener 		= new MouseAdapter() {
													public void mouseEntered(MouseEvent e) {
														((JButton) e.getSource()).setBorder(activeBorder);
													}
													public void mouseExited(MouseEvent e) {
														((JButton) e.getSource()).setBorder(inactiveBorder);
													}
												};
	
	public static ComponentUI createUI(final JComponent c)  {
		c.addContainerListener(new java.awt.event.ContainerAdapter() {
			public void componentAdded(java.awt.event.ContainerEvent e) {
				if (e.getChild() instanceof JButton) enableBorderCycling((JButton)e.getChild());
			}
			public void componentRemoved(java.awt.event.ContainerEvent e) {
				if (e.getChild() instanceof JButton) disableBorderCycling((JButton)e.getChild());
			}
		});
		
		return new ThemeLF_ToolBarUI();
	}

	static void disableBorderCycling(JButton b)  {
		b.setBorder((Border) originalBorders.get(b));
		b.removeMouseListener(mouseListener);
		originalBorders.remove(b);
	}

	static void enableBorderCycling(JButton b)  {
		originalBorders.put(b, b.getBorder());
		
		b.setAlignmentX(JComponent.CENTER_ALIGNMENT);
		b.setAlignmentY(JComponent.CENTER_ALIGNMENT);
		b.setMargin(new Insets(1,1,1,1));
		
		b.setBorder(inactiveBorder);
		b.setRequestFocusEnabled(false); 
		b.addMouseListener(mouseListener);
	}

	public void installUI(JComponent c)  {
		super.installUI(c);
		
		resource 	= ThemeLF.getUI("ToolBar");
		disabler 	= resource.getDisabler("disable");
	}

	public void paint(Graphics g, JComponent c)  {
		super.paint(g, c);
		
		if (! c.isEnabled()) disabler.paint(c, g, 0, 0, 0, 0);
	}

}
